{ AM Software's CheckBox and RadioButton-controls version 1.0 for Delphi 2.x and 3.x
  Copyright  September 1997 by Alexander Meeder

  These controls are the same as the standard Delphi CheckBox and RadioButton except
  for the fact that you can make a tiny bitmap (a glyph) with four (or three for
  AMRadioButton) different glyphs in it to represent the different states of a
  CheckBox/Radiobutton: Unchecked, Checked, Disabled and Grayed.

  I've used a rather unconventional technique just for the efficiency of the controls.
  Because you probably will use more than one TAMCheckBox/TAMRadioButton-descendant
  at a time it's more efficient to share the same glyph stored in one object, than
  a glyph in every instance of the controls.

  This package come with several example Glyphs. When you want to draw them yourself
  follow these steps:
  * every bitmap must contain 3 or 4 glyphs and every glyph must be the same width,
    there are no restrictions for the dimensions of the bitmap as a whole
  * the glyphs must be in the following order: Off/Unchecked, On/Checked,
    Disabled and Grayed (the last only for TAMCheckBox).

  A little enhancement over the Delphi controls is that you can use multi-line
  captions, even with tab-stops of which you can set the indent by using the
  tab-property.

  To use the controls follow these steps:
  1. Place a TAMCbRbHost-component on your form
  2. Place 1 of more TAMCheckBox/TAMRadioButton controls on your form
  3. Select those controls and set the 'Host'-property to refer to the
     TAMCbRbHost-component
  4. Set the desired properties of TAMCbRbHost: Font(In)Active, Glyph, Margin,
                                                NumGlyphs, TransparentColor
  5. Set the desired properties of TAMCheckBox, TAMRadioButton: Tab

  Two extra events:
  MouseEnter
  MouseLeave

  Quastions, suggestions? Mail me at: ameeder@dds.nl
}

unit AMCbRb;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons;

type
  TCheck = (BmpOff, BmpOn, BmpDisabled, BmpGrayed);
  TMargin = 1..25;
  TTab = 1..10;

  TCmMouseEnter = record
    Msg    : Cardinal;
    Unused : Integer;
    Sender : TControl;             
    Result : LongInt;
  end;
  TCmMouseLeave = TCmMouseEnter;

  TAMCbRbHost = class;  // Forward declaration of TAMCbRbHost-class
  TFontPA = class(TPersistent)
  private
    FColor: TColor;
    FStyle: TFontStyles;
    FParent: TAMCbRbHost;

    procedure SetColor(value: TColor);
    procedure SetStyle(value: TFontStyles);
  protected
  public
    property Parent: TAMCbRbHost read FParent write FParent;
  published
    property Color: TColor read FColor write SetColor;
    property Style: TFontStyles read FStyle write SetStyle;
  end;

  TMouseEnter = procedure (Sender: TControl; var Message: TCMMouseEnter) of object;
  TMouseLeave = procedure (Sender: TControl; var Message: TCMMouseLeave) of object;

  TAMCbRbHost = class(TComponent)
  private
    FFontActive,
    FFontInActive: TFontPA;
    FGlyph: TBitmap;
    FMargin: TMargin;
    FNumGlyphs: TNumGlyphs;
    FTransparentColor: TColor;

    procedure SetGlyph(value: TBitmap);
    procedure SetMargin(value: TMargin);
    procedure SetNumGlyphs(value: TNumGlyphs);
    procedure SetTransparentColor(value: TColor);
    function  GetHeight: integer;
    function  GetWidth: integer;
    function  GetSrcRect(index: integer): TRect;
  protected
    procedure UpdateCtrls;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property OffRect: TRect index 0 read GetSrcRect;
    property OnRect: TRect index 1 read GetSrcRect;
    property DisabledRect: TRect index 2 read GetSrcRect;
    property GrayedRect: TRect index 3 read GetSrcRect;
    property Height: integer read GetHeight;
    property Width: integer read GetWidth;
  published
    property FontActive: TFontPA read FFontActive write FFontActive;
    property FontInactive: TFontPA read FFontInactive write FFontInactive;
    property Glyph: TBitmap read FGlyph write SetGlyph;
    property Margin: TMargin read FMargin write SetMargin default 7;
    property NumGlyphs: TNumGlyphs read FNumGlyphs write SetNumGlyphs default 4;
    property TransparentColor: TColor read FTRansparentColor write SetTransparentColor default clOlive;
  end;

  TAMCheckBox = class(TCustomControl)
  private
    FAlignment: TLeftRight;
    FAllowGrayed: boolean;
    FHost: TAMCbRbHost;
    FState: TCheckBoxState;

    FBuffer: TBitmap;

    FActive,
    FChecked,
    Focused,
    HasMouse,
    MouseIsDown: boolean;
    FTab: TTab;

    FMouseEnter : TMouseEnter;
    FMouseLeave : TMouseLeave;

    procedure SetAlignment(value: TLeftRight);
    procedure SetAllowGrayed(value: boolean);
    procedure SetChecked(value: boolean);
    procedure SetHost(value: TAMCbRbHost);
    procedure SetState(value: TCheckBoxState);
    procedure SetTab(value: TTab);

    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MOUSEMOVE;
    procedure CmDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CmEnabledChanged(var Message: TWmNoParams); message CM_ENABLEDCHANGED;
    procedure CmFocusChanged(var Message: TCmFocusChanged); message CM_FOCUSCHANGED;
    procedure CmMouseEnter(var Message: TCmMouseEnter); message CM_MOUSEENTER;
    procedure CmMouseLeave(var Message: TCmMouseLeave); message CM_MOUSELEAVE;
    procedure CmTextChanged(var Message: TWmNoParams); message CM_TEXTCHANGED;
    procedure CmVisibleChanged(var Message: TWmNoParams); message CM_VISIBLECHANGED;
    procedure CnCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure Click; override;
    procedure DblClick; override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Alignment: TLeftRight read FAlignment write SetAlignment default taLeftJustify;
    property AllowGrayed: boolean read FAllowGrayed write SetAllowGrayed default false;
    property Caption;
    property Checked: boolean read FChecked write SetChecked default false;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Hint;
    property Host: TAMCbRbHost read FHost write SetHost;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property State: TCheckBoxState read FState write SetState default cbUnchecked;
    property Tab: TTab read FTab write SetTab default 2;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter : TMouseEnter read FMouseEnter write FMouseEnter;
    property OnMouseLeave : TMouseLeave read FMouseLeave write FMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

  TAMRadioButton = class(TCustomControl)
  private
    FAlignment: TLeftRight;
    FHost: TAMCbRbHost;
    FTab: TTab;

    FBuffer: TBitmap;

    FActive,
    FChecked,
    Focused,
    HasMouse,
    MouseIsDown: boolean;

    FMouseEnter: TMouseEnter;
    FMouseLeave: TMouseLeave;

    procedure SetAlignment(value: TLeftRight);
    procedure SetChecked(value: boolean);
    procedure SetHost(value: TAMCbRbHost);
    procedure SetTab(value: TTab);

    procedure WMMouseMove(var Message: TWMMouseMove); message WM_MOUSEMOVE;
    procedure CmDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
    procedure CmEnabledChanged(var Message: TWmNoParams); message CM_ENABLEDCHANGED;
    procedure CmFocusChanged(var Message: TCmFocusChanged); message CM_FOCUSCHANGED;
    procedure CmMouseEnter(var Message: TCmMouseEnter); message CM_MOUSEENTER;
    procedure CmMouseLeave(var Message: TCmMouseLeave); message CM_MOUSELEAVE;
    procedure CmTextChanged(var Message: TWmNoParams); message CM_TEXTCHANGED;
    procedure CmVisibleChanged(var Message: TWmNoParams); message CM_VISIBLECHANGED;
    procedure CnCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure Click; override;
    procedure DblClick; override;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Alignment: TLeftRight read FAlignment write SetAlignment default taLeftJustify;
    property Caption;
    property Checked: boolean read FChecked write SetChecked default false;
    property Color;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Host: TAMCbRbHost read FHost write SetHost;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Tab: TTab read FTab write SetTab default 2;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseEnter: TMouseEnter read FMouseEnter write FMouseEnter;
    property OnMouseLeave: TMouseLeave read FMouseLeave write FMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

{$R DefChk.res}

// Begin of TAMRadioButton-implementation

procedure TAMRadioButton.Paint;
var
  SrcRect, DstRect, CaptRect: TRect;
  GlyphWidth, GlyphHeight: integer;
  uFormat: UINT;
  DrawTextParams: TDrawTextParams;
begin
  inherited Paint;

  if Assigned(Host) then
  begin
    if Enabled then
       if FChecked then
          SrcRect := Host.OnRect
       else
          SrcRect := Host.OffRect
    else
      SrcRect := Host.DisabledRect;

    GlyphWidth  := Host.Width;
    GlyphHeight := Host.Height;

    case Alignment of
      taLeftJustify  : begin
                         DstRect := Rect(0,0,GlyphWidth,GlyphHeight);
                         OffsetRect(DstRect,1,1);
                       end;
      taRightJustify : begin
                         DstRect := Rect(ClientWidth-GlyphWidth, 0, ClientWidth, GlyphHeight);
                         OffsetRect(DstRect,-1,1);
                       end;
    end;

    FBuffer.Width := Self.Width;
    FBuffer.Height := Self.Height;
    with FBuffer.Canvas do
    begin
      Brush.Color := Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
      BrushCopy(DstRect, Host.Glyph, SrcRect, Host.TransparentColor);

      Font.Assign(Self.Font);
      if HasMouse then
      begin
        Font.Style := Host.FontActive.Style;
        Font.Color := Host.FontActive.Color;
      end
      else
      begin
        Font.Style := Host.FontInactive.Style;
        Font.Color := Host.FontInactive.Color;
      end;
      if Focused then Font.Style := Font.Style+[fsUnderline];
      if not Enabled then Font.Color := clGray;

      DrawTextParams.iTabLength := FTab;
      DrawTextParams.iLeftMargin := 0;
      DrawTextParams.iRightMargin := 0;
      DrawTextParams.cbSize := SizeOf(DrawTextParams);

      case Alignment of
        taLeftJustify  : CaptRect := Rect((GlyphWidth)+Host.Margin, 1, ClientWidth-1, ClientHeight-1);
        taRightJustify : CaptRect := Rect(1, 1, ClientWidth-((GlyphWidth)+Host.Margin), ClientHeight-1);
      end;
      uFormat := DT_CALCRECT or DT_WORDBREAK or DT_EXPANDTABS or DT_END_ELLIPSIS;
      DrawTextEx(Handle, PChar(Caption), Length(Caption), CaptRect, uFormat, @DrawTextParams);
      uFormat := DT_WORDBREAK or DT_EXPANDTABS or DT_END_ELLIPSIS or DT_TABSTOP;
      DrawTextEx(Handle, PChar(Caption), Length(Caption), CaptRect, uFormat, @DrawTextParams);
      Self.Canvas.Draw(0,0,FBuffer);
    end;
  end
  else
  begin
    with Canvas do
    begin
      Font := Self.Font;
      Brush.Style := bsClear;
      Pen.Style := psDot;
      TextOut((ClientRect.Right-TextWidth(Caption)) div 2,(ClientRect.Bottom-TextHeight(Caption)) div 2,Caption);
      Canvas.Rectangle(ClientRect.Left,ClientRect.Top,ClientRect.Right,ClientRect.Bottom);
    end;
  end
end;

procedure TAMRadioButton.SetAlignment(value: TLeftRight);
begin
  if value <> FAlignment then
  begin
    FAlignment := value;
    Repaint;
  end;
end;

procedure TAMRadioButton.SetChecked(Value: Boolean);

  procedure TurnSiblingsOff;
  var
    I: Integer;
    Sibling: TControl;
  begin
    if Parent <> nil then
      with Parent do
        for I := 0 to ControlCount - 1 do
        begin
          Sibling := Controls[I];
          if (Sibling <> Self) and (Sibling is TAMRadioButton) then
             TAMRadioButton(Sibling).SetChecked(False);
        end;
  end;

begin
  if FChecked <> Value then
  begin
    FChecked := Value;
    if Value then TurnSiblingsOff;
    Repaint;
  end;
end;

procedure TAMRadioButton.SetHost(value: TAMCbRbHost);
begin
  if value <> FHost then
  begin
    FHost := value;
    Repaint;
  end;
end;

procedure TAMRadioButton.SetTab(value: TTab);
begin
  if value <> FTab then
  begin
    FTab := value;
    Repaint;
  end;
end;

procedure TAMRadioButton.WMMouseMove(var Message: TWMMouseMove);
var
  CurrentPoint: TPoint;
begin
  CurrentPoint.X := Message.XPos;
  CurrentPoint.Y := Message.YPos;
  if not HasMouse and PtInRect(GetClientRect, CurrentPoint) then
  begin
    HasMouse := true;
    Repaint;
  end;
  inherited;
end;

procedure TAMRadioButton.CmDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus then
    begin
      Click;
      Result := 1;
    end
    else
      inherited;
end;

procedure TAMRadioButton.CmEnabledChanged(var Message: TWmNoParams);
begin
  inherited;
  Repaint;
end;

procedure TAMRadioButton.CmFocusChanged(var Message: TCmFocusChanged);
begin
  with Message do
  if Sender is TAMRadioButton then FActive := Sender = Self;
  inherited;
end;

procedure TAMRadioButton.CmMouseEnter(var Message: TCmMouseEnter);
begin
  inherited;
  if Enabled then
  begin
    HasMouse := true;
    Repaint;
  end;
  if Assigned(FMouseEnter) then FMouseEnter(Self, Message);
end;

procedure TAMRadioButton.CmMouseLeave(var Message: TCmMouseLeave);
begin
  inherited;
  if Enabled then
  begin
    HasMouse := false;
    Repaint;
  end;
  if Assigned(FMouseLeave) then FMouseLeave(Self, Message);
end;

procedure TAMRadioButton.CmTextChanged(var Message: TWmNoParams);
begin
  inherited;
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle - [csOpaque];
  Repaint;
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle + [csOpaque];
end;

procedure TAMRadioButton.CmVisibleChanged(var Message: TWmNoParams);
begin
  inherited;
  Repaint;
end;

procedure TAMRadioButton.CnCommand(var Message: TWMCommand);
begin
  if Message.NotifyCode = BN_CLICKED then Click;
end;

procedure TAMRadioButton.Click;
begin
  inherited Click;
  Checked := True;
  Repaint;
end;

procedure TAMRadioButton.DblClick;
begin
  inherited DblClick;
  Click;
end;

procedure TAMRadioButton.DoEnter;
begin
  inherited DoEnter;
  Focused := True;
  Repaint;
end;

procedure TAMRadioButton.DoExit;
begin
  inherited DoExit;
  Focused := False;
  Repaint;
end;

procedure TAMRadioButton.KeyUp(var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_SPACE) then
  begin
    Click;
  end;
  inherited KeyUp(Key, Shift);
end;

procedure TAMRadioButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then
  begin
    SetFocus;
    MouseIsDown := true;
    Repaint;
    inherited MouseDown(Button, Shift, X, Y);
  end;
end;

procedure TAMRadioButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then
  begin
    MouseIsDown := false;
    Repaint;
    inherited MouseUp(Button, Shift, X, Y);
  end;
end;

procedure TAMRadioButton.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FHost then FHost := nil;
  end;
end;

constructor TAMRadioButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle + [csOpaque, csSetCaption];
  SetBounds(0,0,97,17);
  FBuffer := TBitmap.Create;
  FBuffer.Width := Width;
  FBuffer.Height := Height;
  FAlignment := taLeftJustify;
  FChecked := false;
  FTab := 2;
end;

destructor TAMRadioButton.Destroy;
begin
  FBuffer.Free;
  inherited Destroy;
end;

// Begin of TAMCheckBox-implementation

procedure TAMCheckBox.Paint;
var
  SrcRect, DstRect, CaptRect: TRect;
  GlyphWidth, GlyphHeight: integer;
  uFormat: UINT;
  DrawTextParams: TDrawTextParams;
begin
  inherited Paint;

  if Assigned(Host) then
  begin
    if Enabled then
    begin
      case FState of
        cbUnchecked : SrcRect := Host.OffRect;
        cbChecked   : SrcRect := Host.OnRect;
        cbGrayed    : SrcRect := Host.GrayedRect;
      end;
    end
    else
      SrcRect := Host.DisabledRect;

    GlyphWidth  := Host.Width;
    GlyphHeight := Host.Height;

    case Alignment of
      taLeftJustify  : begin
                         DstRect := Rect(0,0,GlyphWidth,GlyphHeight);
                         OffsetRect(DstRect,1,1);
                       end;
      taRightJustify : begin
                         DstRect := Rect(ClientWidth-GlyphWidth, 0, ClientWidth, GlyphHeight);
                         OffsetRect(DstRect,-1,1);
                       end;
    end;

    FBuffer.Width := Self.Width;
    FBuffer.Height := Self.Height;
    with FBuffer.Canvas do
    begin
      Brush.Color := Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect);
      BrushCopy(DstRect, Host.Glyph, SrcRect, Host.TransparentColor);

      Font.Assign(Self.Font);
      if HasMouse then
      begin
        Font.Style := Host.FontActive.Style;
        Font.Color := Host.FontActive.Color;
      end
      else
      begin
        Font.Style := Host.FontInactive.Style;
        Font.Color := Host.FontInactive.Color;
      end;
      if Focused then Font.Style := Font.Style+[fsUnderline];
      if not Enabled then Font.Color := clGray;

      DrawTextParams.iTabLength := FTab;
      DrawTextParams.iLeftMargin := 0;
      DrawTextParams.iRightMargin := 0;
      DrawTextParams.cbSize := SizeOf(DrawTextParams);

      case Alignment of
        taLeftJustify  : CaptRect := Rect((GlyphWidth)+Host.Margin, 1, ClientWidth-1, ClientHeight-1);
        taRightJustify : CaptRect := Rect(1, 1, ClientWidth-((GlyphWidth)+Host.Margin), ClientHeight-1);
      end;
      uFormat := DT_CALCRECT or DT_WORDBREAK or DT_EXPANDTABS or DT_END_ELLIPSIS;
      DrawTextEx(Handle, PChar(Caption), Length(Caption), CaptRect, uFormat, @DrawTextParams);
      uFormat := DT_WORDBREAK or DT_EXPANDTABS or DT_END_ELLIPSIS or DT_TABSTOP;
      DrawTextEx(Handle, PChar(Caption), Length(Caption), CaptRect, uFormat, @DrawTextParams);
      Self.Canvas.Draw(0,0,FBuffer);
    end;
  end
  else
  begin
    with Canvas do
    begin
      Font := Self.Font;
      Brush.Style := bsClear;
      Pen.Style := psDot;
      TextOut((ClientRect.Right-TextWidth(Caption)) div 2,(ClientRect.Bottom-TextHeight(Caption)) div 2,Caption);
      Canvas.Rectangle(ClientRect.Left,ClientRect.Top,ClientRect.Right,ClientRect.Bottom);
    end;
  end;
end;

procedure TAMCheckBox.SetAlignment(value: TLeftRight);
begin
  if value <> FAlignment then
  begin
    FAlignment := value;
    Repaint;
  end;
end;

procedure TAMCheckBox.SetAllowGrayed(value: boolean);
begin
  if value <> FAllowGrayed then
  begin
    FAllowGrayed := value;
    if not value then State := cbUnchecked;
  end;
end;

procedure TAMCheckBox.SetChecked(value: boolean);
begin
  if (value <> FChecked) and Enabled then
  begin
    FChecked := value;
    if value then FState := cbChecked else FState := cbUnchecked;
    Repaint;
  end;
end;

procedure TAMCheckBox.SetHost(value: TAMCbRbHost);
begin
  FHost := value;
  Repaint;
end;

procedure TAMCheckBox.SetState(value: TCheckBoxState);
begin
  if (value = cbGrayed) and not FAllowGrayed then Exit;
  if (value <> FState) and Enabled then
  begin
    FState := value;
    FChecked := FState = cbChecked;
    Repaint;
  end;
end;

procedure TAMCheckBox.SetTab(value: TTab);
begin
  if value <> FTab then
  begin
    FTab := value;
    Repaint;
  end;
end;

procedure TAMCheckBox.WMMouseMove(var Message: TWMMouseMove);
var
  CurrentPoint: TPoint;
begin
  CurrentPoint.X := Message.XPos;
  CurrentPoint.Y := Message.YPos;
  if not HasMouse and PtInRect(GetClientRect, CurrentPoint) then
  begin
    HasMouse := true;
    Repaint;
  end;
  inherited;
end;

procedure TAMCheckBox.CmDialogChar(var Message: TCMDialogChar);
begin
  with Message do
    if IsAccel(CharCode, Caption) and CanFocus then
    begin
      Click;
      Result := 1;
    end
    else
      inherited;
end;

procedure TAMCheckBox.CmEnabledChanged(var Message: TWmNoParams);
begin
  inherited;
  Repaint;
end;

procedure TAMCheckBox.CmFocusChanged(var Message: TCmFocusChanged);
begin
  with Message do
  if Sender is TAMCheckBox then FActive := Sender = Self;
  inherited;
end;

procedure TAMCheckBox.CmMouseEnter(var Message: TCmMouseEnter);
begin
  inherited;
  if Enabled then
  begin
    HasMouse := true;
    Repaint;
  end;
  if Assigned(FMouseEnter) then FMouseEnter(Self, Message);
end;

procedure TAMCheckBox.CmMouseLeave(var Message: TCmMouseLeave);
begin
  inherited;
  if Enabled then
  begin
    HasMouse := false;
    Repaint;
  end;
  if Assigned(FMouseLeave) then FMouseLeave(Self, Message);
end;

procedure TAMCheckBox.CmTextChanged(var Message: TWmNoParams);
begin
  inherited;
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle - [csOpaque];
  Repaint;
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle + [csOpaque];
end;

procedure TAMCheckBox.CmVisibleChanged(var Message: TWmNoParams);
begin
  inherited;
  Repaint;
end;

procedure TAMCheckBox.CnCommand(var Message: TWMCommand);
begin
  if Message.NotifyCode = BN_CLICKED then Click;
end;

procedure TAMCheckBox.Click;
begin
  inherited Click;
  if AllowGrayed then
    case FState of
      cbUnchecked : FState := cbGrayed;
      cbGrayed    : FState := cbChecked;
      cbChecked   : FState := cbUnchecked;
    end
  else
    case FState of
      cbUnchecked : FState := cbChecked;
      cbChecked   : FState := cbUnchecked;
    end;
  Repaint;
end;

procedure TAMCheckBox.DblClick;
begin
  inherited DblClick;
  Click;
end;

procedure TAMCheckBox.DoEnter;
begin
  inherited DoEnter;
  Focused := True;
  Repaint;
end;

procedure TAMCheckBox.DoExit;
begin
  inherited DoExit;
  Focused := False;
  Repaint;
end;

procedure TAMCheckBox.KeyUp(var Key: Word; Shift: TShiftState);
begin
  if (Key = VK_SPACE) then
  begin
    Click;
  end;
  inherited KeyUp(Key, Shift);
end;

procedure TAMCheckBox.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then
  begin
    SetFocus;
    MouseIsDown := true;
    Repaint;
    inherited MouseDown(Button, Shift, X, Y);
  end;
end;

procedure TAMCheckBox.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and Enabled then
  begin
    MouseIsDown := false;
    Repaint;
    inherited MouseUp(Button, Shift, X, Y);
  end;
end;

procedure TAMCheckBox.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Operation = opRemove then
  begin
    if AComponent = FHost then FHost := nil;
  end;
end;

constructor TAMCheckBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) then ControlStyle := ControlStyle + [csOpaque, csSetCaption];
  SetBounds(0,0,97,17);
  FBuffer := TBitmap.Create;
  FBuffer.Width := Self.Width;
  FBuffer.Height := Self.Height;
  FAlignment := taLeftJustify;
  FAllowGrayed := false;
  FChecked := false;
  FState := cbUnchecked;
  FTab := 2;
  TabStop := true;
end;

destructor TAMCheckBox.Destroy;
begin
  FBuffer.Free;
  inherited Destroy;
end;

// Begin of help controls...

procedure TAMCbRbHost.UpdateCtrls;
var
  X: integer;
begin
  for X := 0 to (Owner as TForm).ControlCount-1 do
  begin
    if (Owner as TForm).Controls[X] is TAMCheckBox then
      if ((Owner as TForm).Controls[X] as TAMCheckBox).Host = self then
         (Owner as TForm).Controls[X].Repaint;
    if (Owner as TForm).Controls[X] is TAMRadioButton then
      if ((Owner as TForm).Controls[X] as TAMRadioButton).Host = self then
         (Owner as TForm).Controls[X].Repaint;
  end;
end;

function TAMCbRbHost.GetSrcRect(index: integer): TRect;
begin
  Result.Left := index * (FGlyph.Width div FNumGlyphs);
  Result.Top := 0;
  Result.Right := (index+1) * (FGlyph.Width div FNumGlyphs);
  Result.Bottom := FGlyph.Height;
end;

procedure TAMCbRbHost.SetGlyph(value: TBitmap);
begin
  FGlyph.Assign(Value);
  if (FGlyph.Empty = true) then FGlyph.Handle := LoadBitmap(hInstance, 'DEFCHK');
  UpdateCtrls;
end;

procedure TAMCbRbHost.SetMargin(value: TMargin);
begin
  if value <> FMargin then
  begin
    FMargin := value;
    UpdateCtrls;
  end;
end;

procedure TAMCbRbHost.SetNumGlyphs(value: TNumGlyphs);
begin
  if value <> FNumGlyphs then
  begin
    FNumGlyphs := value;
    UpdateCtrls;
  end;
end;

procedure TAMCbRbHost.SetTransparentColor(value: TColor);
begin
  if value <> FTransparentColor then
  begin
    FTransparentColor := value;
    UpdateCtrls;
  end;
end;

function TAMCbRbHost.GetHeight: integer;
begin
  Result := FGlyph.Height;
end;

function TAMCbRbHost.GetWidth: integer;
begin
  Result := FGlyph.Width div FNumGlyphs;
end;

constructor TAMCbRbHost.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FGlyph := TBitmap.Create;
  FGlyph.Handle := LoadBitmap(hInstance, 'DEFCHK');
  FMargin := 7;
  FNumGlyphs := 4;
  FFontActive := TFontPA.Create;
  FFontActive.Parent := Self;
  FFontActive.Color := clBlue;
  FFontActive.Style := [];
  FFontInactive := TFontPA.Create;
  FFontInactive.Parent := Self;
  FFontInactive.Color := clBlack;
  FFontInactive.Style := [];
  FTransparentColor := clOlive;
end;

destructor TAMCbRbHost.Destroy;
begin
  FFontActive.Free;
  FFontInActive.Free;
  inherited Destroy;
end;

// Begin of TFontPA-implementation...

procedure TFontPA.SetColor(value: TColor);
begin
  if value <> FColor then
  begin
    FColor := value;
    Parent.UpdateCtrls;
  end;
end;

procedure TFontPA.SetStyle(value: TFontStyles);
begin
  if value <> FStyle then
  begin
    FStyle := value;
    Parent.UpdateCtrls;
  end;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TAMCbRbHost, TAMCheckBox, TAMRadioButton]);
end;

end.
